import processing.core.*; 
import processing.xml.*; 

import processing.serial.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class TPL0501_GUI extends PApplet {

/************************************************************
*  TPL0401 GUI                                              *
*                                                           *
*  Author: Chris Kraft                                      *
*                                                           *
*  Date: July 2011                                          *
*                                                           *
*  To change the values of the DPOTs, simple serial flags   *
*  are used to facilitate communication between the         *
*  LaundPad and the computer.                               *
************************************************************/



/************************************************************
*  Constant global variables                                *
************************************************************/
final float PERCENT_STEP = 1.f/256.f;
final float RSTEP = 100000.f/256.f;                           // Resistance increase of each tap on the TPL0401
final char AFLAG = 0x65;                                   // Serial flag for TPL0401A RX data
final char BFLAG = 0x42;                                   // Serial flag for TPL0401B RX data
final char MODE_FLAG = 0x02;                               // Serial flag for mode change
final char TX_A_FLAG = 0x61;
final char TX_B_FLAG = 0x6A;
final char TX_RESET_FLAG = 0x7F;

/************************************************************
*  Global variables                                         *
************************************************************/
boolean LaunchPadComPortFound=false;                       
boolean percent = false;
PFont font;
PImage backgroundImage;                                   // Background Image located in \data folder

boolean deciFlag = false;
int fontX=210, fontY=313;                                   // Font locations for number input
int linecount=0;
ImageButtons rect1, tapb, percentb, resistance, aselected, aunselected, bselected, bunselected;                                // Button definition for TPL0401A and TPL0401B buttons

Serial LaunchPad;                                         

float deci = 10;
int [] keyIn= new int[7];                                // Location to store keyboard input buffer
int keyIndex=0;                                           // Pointer associated with input buffer

boolean invalidinput=false;                               // If keyboard input is larger than 10,000 this will flag as true

boolean buttonpressA=false, buttonpressB=false, buttonpressC=false;           // Flags for if a button was pressed
boolean nokeypress=true;                                  // Flag for if no button was recently pressed

float resistorValue;                                        // Resistor value closest to keyboard input 

char flag=0;                                              // Stored serial flag location
char data;                                                // Stored serial data location
char potFlag = TX_A_FLAG;
int Adata=0, Bdata=0;                                     // Potentiometer Tap value
int POTAres=0,POTBres=0;                                            // Potentiometer resistor value

float POTAPercent=0,POTBPercent=0;

int wiper=0;                                              // Tap value

boolean first= true;
boolean LEDmode = false;

boolean A_selected = true;

public void setup()
{
  size(600,600);
  smooth();
  background(0,0,0);
  backgroundImage = loadImage("background.PNG");                  // Load \data\background.PNG
  //font = loadFont("CourierNew36.vlw");
  //textFont(font,32);
  
  // Load Images used in creating the image buttons
  //PImage c = loadImage("tpl0401.PNG");                            
  //PImage r = loadImage("tpl0401hover.PNG");
  //PImage d = loadImage("tpl0401selected.PNG");
  
  PImage res = loadImage("resistance.PNG");
  PImage res_hover = loadImage("resistancehover.PNG");
  PImage res_selected = loadImage("resistanceselected.PNG");

  PImage per = loadImage("percent.PNG");
  PImage per_hover = loadImage("percenthover.PNG");
  PImage per_selected = loadImage("percentselected.PNG");
  
  PImage tap = loadImage("tap.PNG");
  PImage tap_hover = loadImage("taphover.PNG");
  PImage tap_selected = loadImage("tapselected.PNG");
  
  PImage a = loadImage("A.png");
  PImage ahover = loadImage("ahover.PNG");
  PImage ahover2 = loadImage("Ahover2.PNG");
  PImage aselect = loadImage("aselect.PNG"); 
  
  PImage b = loadImage("B.PNG");
  PImage bhover = loadImage("Bhover.PNG");
  PImage bhover2 = loadImage("Bhover2.PNG");
  PImage bselect = loadImage("Bselect.PNG");
  // Create the two image buttons associated with TPL0401A/B
  //rect1 = new ImageButtons(75,292,b.width,b.height,b,r,d);
  //rect2 = new ImageButtons(202,421,b.width,b.height,b,r,d);
  resistance = new ImageButtons(240,156,res.width,res.height,res,res_hover,res_selected);
  tapb = new ImageButtons(130,120,tap.width,tap.height,tap,tap_hover,tap_selected);
  percentb = new ImageButtons(391,157,per.width,per.height,per,per_hover,per_selected);  
  
  aselected = new ImageButtons(130, 70, b.width,b.height,bselect,bhover2,b);
  aunselected = new ImageButtons(130, 70, b.width,b.height,b,bhover,bselect);

  bselected = new ImageButtons(350, 70, a.width,a.height,aselect,ahover2,a);
  bunselected = new ImageButtons(350, 70, a.width,a.height,a,ahover,aselect);
  
  //LaunchPad = new Serial(this, Serial.list()[2],9600);          //Remove the .NET requirement by uncommenting this line 
                                                                  //(and selecting the correct port)
                                                                  
  findLaunchPad();                                                //Comment this line to remove .NET requirement
}

public void draw()
{
  if (LaunchPadComPortFound)                                      //Comment this line to remove .NET requirement
  {                                                               //Comment this line to remove .NET requirement
    if(first)
    {
      LaunchPad.write(0x7F);
      first=false; 
    }
    if(!(buttonpressA||buttonpressB||buttonpressC))
    {
      if(!LEDmode)
      {
        image(backgroundImage,0,0);                               // Set \data\background.PNG as background image
        textSize(16);                                             // Define global text size
        fill(0);                                                  // black fill

        // Update and display image buttons
        //rect1.update();
        //rect1.display();
        resistance.update();
        tapb.update();
        if(A_selected){
          percentb.update();
        }
        resistance.display();
        tapb.display();
        if(A_selected){
          percentb.display();
        }
        
        if(A_selected)
        {
            aselected.update();
            bunselected.update();
            aselected.display();
            bunselected.display();
        }
        else
        {
            aunselected.update();
            bselected.update();
            aunselected.display();
            bselected.display();
        }
        
        //rect2.update();
        //rect2.display();
        
        // Print potentiometer resistance values along with an Ohm symbol
        if(A_selected)
        {
          text(POTAres + "\u2126",255,193);
          text(Adata,136,156);
          text(binary(Adata,8),111,193);
          text("0x" + hex(Adata,2), 132, 235);
          text(nf(POTAPercent,1,2) + "\u0025", 398, 193);
        }
        else
        {
          text(POTBres + "\u2126",255,193);
          text(Bdata,136,156);
          text(binary(Bdata,8),111,193);
          text("0x" + hex(Bdata,2), 132, 235);
          //text(nf(POTBPercent,1,2) + "\u0025", 398, 193);
        }
                  //text(POTBres + "\u2126",85,476);
        
        // Print potentiometer tap values
       
        
        text("Click on a variable to change its value",90,30);
        if(mousePressed)
        {
           
           if(A_selected)
           {
             
             if(bunselected.pressed)
             {
               potFlag = TX_B_FLAG;
               A_selected = false;
               //LaunchPad.write(0x3C);
             }
           }
         else
        { 
          if(aunselected.pressed)
           {
              A_selected = true;
              potFlag = TX_A_FLAG;
             // LaunchPad.write(0x3C);
           }
        }
           
          
          
           if(percentb.pressed)
           {
               LaunchPad.write(potFlag);
               buttonpressA=true;
           }
           else if (tapb.pressed)
           {
               LaunchPad.write(potFlag);
               buttonpressB = true; 
             
           }
           else if (resistance.pressed)
           {
              LaunchPad.write(potFlag);
              buttonpressC = true;             
             
           }


        }
      }
      else                                                                  // If in LED mode tell user to switch to GUI mode
      {
        background(255);
        fill(0);
        text("Press up and down buttons to enter GUI mode",40,60);     
      
      }
    }
   
    else if(buttonpressC && nokeypress && !invalidinput && !LEDmode)    //  If a button has been pressed 
    {
      linecount++;
      background(255);
      
      textSize(28);
      text("Input Resistance Value 0-100,000\u2126\nand Press Enter",40,60);
      
      fill(255);
      stroke(0);
      rect(200,275,200,50); 
      if (linecount <= 30)
        line(fontX+2,fontY-25, fontX+2, fontY+1);
      stroke(255);
      if (linecount > 30)
      {
        rect(fontX+2,fontY-25, 2, 26);
        if (linecount >= 60)
          linecount = 0; 
      }
      stroke(0);
      fill(0);
      
      text("\u2126",403,313);
      
    }
    
    else if(buttonpressB && nokeypress && !invalidinput && !LEDmode)    //  If a button has been pressed 
    {
      linecount++;
      background(255);
      
      textSize(28);
      text("Input tap value 0 - 255\nand Press Enter",40,60);
      
      fill(255);
      stroke(0);
      rect(200,275,200,50); 
      if (linecount <= 30)
        line(fontX+2,fontY-25, fontX+2, fontY+1);
      stroke(255);
      if (linecount > 30)
      {
        rect(fontX+2,fontY-25, 2, 26);
        if (linecount >= 60)
          linecount = 0; 
      }
      stroke(0);
      fill(0);
      
      text("\u2126",403,313);
      
    }
    
    else if(buttonpressA && nokeypress && !invalidinput && !LEDmode)    //  If a button has been pressed 
    {
      linecount++;
      background(255);
      
      textSize(28);
      text("Input Percentage Value to pass\nand Press Enter",40,60);
      
      fill(255);
      stroke(0);
      rect(200,275,200,50); 
      if (linecount <= 30)
        line(fontX +2,fontY-25, fontX+2, fontY+1);
      stroke(255);
      if (linecount > 30)
      {
        rect(fontX+2,fontY-25, 2, 26);
        if (linecount >= 60)
          linecount = 0; 
      }
      stroke(0);
      fill(0);
      
      text("\u0025",403,313);
      
    }
    
    else if (invalidinput)
    {
       if(keyPressed)
         invalidinput = false; 
      
    }
    else if((buttonpressA||buttonpressB||buttonpressC) && !nokeypress)
    {
      fill(255);
      linecount++;
      
      if (linecount <= 30)
        line(fontX+2,fontY-25, fontX+2, fontY+1);
      stroke(255);
      if (linecount > 30)
      {
        rect(fontX+2,fontY-25, 2, 26);
        if (linecount >= 60)
          linecount = 0; 
      }
      stroke(0);
      fill(0);
    }
    
  }                                                       //Comment this line to remove .NET requirement
  else                                                    //Comment this line to remove .NET requirement
    findLaunchPad();                                      //Comment this line to remove .NET requirement
}

public void keyTyped()
{
  if((buttonpressA||buttonpressB||buttonpressC) && !invalidinput && !LEDmode)
  {
     nokeypress=false;
     if (key !=10)                                                // Key press is not Enter
     {
       if(key!=8)                                                 // Key press is not backspace
       { 
         if (key <58 && key > 47 || key==46)                                 // Key press is between ASCII codes for 0-9
         {
           if (keyIndex < 7)                                      // keyIndex is less than maximum matrix size
           {

              keyIn[keyIndex] = key-48;                                        // key-48 will give the numerical value of the key presse

                
              fill(255);
              stroke(255);
              rect(fontX+2,fontY-25, 2, 26);
              fill(0);
              stroke(0);
              if(keyIn[keyIndex] != -2)
              {
                text(keyIn[keyIndex],fontX,fontY);              // display the key typed and incriment the x pointer location
                fontX+=15;
              }
              else
              {
                text(".",fontX,fontY);
                fontX+=7; 
              }
              
              keyIndex++;                                         // incriment index value
           }
         }
       }
       else                                                        // Backspace was pressed
       { 
        fill(255);                                                // Set fill to white
        noStroke();                                               // Set shapes to have no border
        if(keyIndex > 0)                                             // Make sure there is something to delete
        {
          if(keyIn[keyIndex -1] != -2)
            fontX-=15;                         // decriment the x pointer location
          else
            fontX-=7;
          rect(fontX,fontY-32,390-fontX,40);                             // place a white filled rectangle over the number you want to delete
          keyIndex--;                                             // decriment the index value
          
        }
        fill(0);                                                  // Set color back to black
             
       }
     }
     else                                                         // Enter was pressed
     {
       resistorValue=0;                                           
       for(int i =0; i< keyIndex; i++)
       {                                                 // Change array into a number
         if(keyIn[i] != -2 & !deciFlag)
           resistorValue = resistorValue * 10 + keyIn[i];
         else if (deciFlag)
         {
           if(keyIn[i] == -2)
           {          
             keyPressed = false;
             invalidinput = true;
             fontX=210;                                                // Reset x location
             //fontY=90;
             keyIndex=0;                                              // reset keyindex
             nokeypress=true; 
             deciFlag = false;
           }
           resistorValue = resistorValue + keyIn[i]/deci; 
           deci= deci*10;
         }
           
         else if (keyIn[i] == -2)
         {
            deciFlag = true;   
            deci = 10;
         }
       }
       deciFlag = false;   
       println(resistorValue);                                    // Print resistor value to consoloe for debugging purposes
       if(buttonpressA)
       {
       if( resistorValue > 100 || resistorValue < 0 || invalidinput)            // Resistor value outside of range of TPL0401
       {
         text("Invalid Input, press any key",40,200);             
         keyPressed = false;
         invalidinput = true;
         fontX=210;                                                // Reset x location
         //fontY=90;
         keyIndex=0;                                              // reset keyindex
         nokeypress=true;
        }
        else                                                        // input resistor value is within range of TPL0401
        {
         if(resistorValue == 100)
         {
           LaunchPad.write(0xFF);
         }
         else
         {
         float tempwiper = resistorValue/100.f;

         tempwiper = tempwiper/PERCENT_STEP;
         println("temp: " + tempwiper);
         wiper = round(tempwiper);                                      // wiper value rounds off temp
         println(wiper);
         //float tempres = float(wiper) * RSTEP;          // temp updated to reflect the actual resistance achieved
         

         //POTAres= int(tempres);         // round temp and assign value to POTA or POTB
         //POTAPercent = wiper/128.*100.;
         LaunchPad.write(PApplet.parseByte(wiper));
         }
         buttonpressA=false;                                        // Reset flags and return to initial state
         buttonpressB=false; 
         buttonpressC=false;
         nokeypress=true;
         fontX=210;
         //fontY=90;
         keyIndex=0;
        }
         
       }
       else if (buttonpressB)
       {
       if( resistorValue > 255 || resistorValue < 0 || invalidinput)            // Resistor value outside of range of TPL0401
       {
         text("Invalid Input, press any key",40,200);             
         keyPressed = false;
         invalidinput = true;
         fontX=210;                                                // Reset x location
         //fontY=90;
         keyIndex=0;                                              // reset keyindex
         nokeypress=true;
        }
        else                                                        // input resistor value is within range of TPL0401
        {
         
         
         //float tempres = resistorValue * RSTEP;          // temp updated to reflect the actual resistance achieved


         //POTAres= int(tempres);         // round temp and assign value to POTA or POTB
         //POTAPercent = resistorValue/128.*100.;
         LaunchPad.write(PApplet.parseByte(resistorValue));
         buttonpressA=false;                                        // Reset flags and return to initial state
         buttonpressB=false; 
         buttonpressC= false;
         nokeypress=true;
         fontX=210;
         //fontY=90;
         keyIndex=0;
        }
         
       }
       else if (buttonpressC)
       {
       if( resistorValue > 100000 || resistorValue < 0 || invalidinput)            // Resistor value outside of range of TPL0401
       {
         text("Invalid Input, press any key",40,200);             
         keyPressed = false;
         invalidinput = true;
         fontX=210;                                                // Reset x location
         //fontY=90;
         keyIndex=0;                                              // reset keyindex
         nokeypress=true;
        }
        else                                                        // input resistor value is within range of TPL0401
        {
         if(resistorValue == 100000)
         {
           LaunchPad.write(0xFF);
         }
         else
         {
         float temp = resistorValue/RSTEP;
         
        wiper = round(temp);   
        println(wiper);
         
        LaunchPad.write(wiper);
         }
        //temp = float(wiper) * RSTEP;

        //POTAres= int(temp);
        //POTAPercent = float(wiper)/128.*100.;
         
         
        buttonpressA=false;
        buttonpressB=false; 
        buttonpressC= false;
        nokeypress=true;
        fontX=210;
         
        keyIndex=0;
        }
         
       }
       
     }
     
   } 
}

public void serialEvent(Serial LaunchPad)
{
  if (LaunchPad.available()>2)                                      // If more than 1 byte in serial buffer
  {
    flag = LaunchPad.readChar();                                    // Flag is always sent first from MSP430
    println(PApplet.parseInt(flag));
    char bytesize = LaunchPad.readChar();
    data = LaunchPad.readChar();                                    // Data comes after flag from MSP430
    println(PApplet.parseInt(data));
    switch(flag)
    {
        case AFLAG: if(bytesize == 0x33)
                      Adata = PApplet.parseInt(data) + 128;
                    else
                      Adata = PApplet.parseInt(data);                              // Aflag recieved set Adata equal to data byte
                    float tempres = Adata * RSTEP;          // temp updated to reflect the actual resistance achieved
                    POTAres= PApplet.parseInt(tempres);         // round temp and assign value to POTA or POTB
                    POTAPercent = Adata/256.f*100.f;
                    
                    
                    //println (Adata);
        break;
        case BFLAG: if(bytesize == 0x33)
                      Bdata = PApplet.parseInt(data) + 128;
                    else
                      Bdata = PApplet.parseInt(data);                              // Bflag recieved set Bdata equal to data byte
                    float tempres1 = Bdata * RSTEP;          // temp updated to reflect the actual resistance achieved
                    POTBres= PApplet.parseInt(tempres1);         // round temp and assign value to POTA or POTB
                    //POTBPercent = Bdata/256.*100.;
                   // println (Bdata);
        break;
        /*case MODE_FLAG:  if (data == 0x03)                               // Mode flag recieved update LEDmode Value
                           LEDmode = false;
                         else if (data == 0x01)
                           LEDmode = true;
        break;
        default: Adata = 0;
                 Bdata = 0;
        break; */
    }
  }
}

class Button
{
  int x, y;
  int w, h;
  int basecolor, highlightcolor;
  int currentcolor;
  boolean over = false;
  boolean pressed = false;   
  
  public void pressed() {
    if(over && mousePressed) {
      pressed = true;
    } else {
      pressed = false;
    }    
  }
  
  public boolean overRect(int x, int y, int width, int height) {
  if (mouseX >= x && mouseX <= x+width && 
      mouseY >= y && mouseY <= y+height) {
    return true;
  } else {
    return false;
  }
}
}

class ImageButtons extends Button 
{
  PImage base;
  PImage roll;
  PImage down;
  PImage currentimage;

  ImageButtons(int ix, int iy, int iw, int ih, PImage ibase, PImage iroll, PImage idown) 
  {
    x = ix;
    y = iy;
    w = iw;
    h = ih;
    base = ibase;
    roll = iroll;
    down = idown;
    currentimage = base;
  }
  
  public void update() 
  {
    over();
    pressed();
    if(pressed) {
      currentimage = down;
    } else if (over){
      currentimage = roll;
    } else {
      currentimage = base;
    }
  }
  
  public void over() 
  {
    if( overRect(x, y, w, h) ) {
      over = true;
    } else {
      over = false;
    }
  }
  
  public void display() 
  {
    image(currentimage, x, y);
  }
}


public void findLaunchPad()                          //Comment this function to remove .NET requirement
{
  String ComPortName ="";  
  try
  {
    Process proc = Runtime.getRuntime().exec("FindAppUART.exe"); 
    proc.waitFor();
    int exitVal = proc.exitValue();
    // Get the first line from the process' STDOUT
    BufferedReader buf = new BufferedReader(new InputStreamReader(proc.getInputStream()));
    ComPortName = buf.readLine();    
    if (ComPortName.substring(0,3).equals("COM") != true)
      ComPortName = "";
    else
    {
      LaunchPadComPortFound = true;  
       LaunchPad = new Serial(this, ComPortName, 9600);
     println(ComPortName);  
    }
  }
  catch(Exception e)
  {
    println(e);    
  }   
}
  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#D4D0C8", "TPL0501_GUI" });
  }
}
